<?php

if (!class_exists('ESIG_GF_VALUE')):

    abstract class ESIG_GF_VALUE {

        public static function generate_value($formid, $field_id, $entry_id, $docId, $display = "value", $option = "default") {

            $form = GFAPI::get_form($formid);
           
            $field = GFFormsModel::get_field($form, $field_id);
            
            $lead = ESIG_GF_SETTINGS::getEntry($docId);
            
            if (!$lead) {

                $lead = GFAPI::get_entry($entry_id);

            }

            if (is_wp_error($lead)) {

                return false;

            }

            $value = self::get_default_value($lead, $field, $form, $field_id, $display, $option);

            return $value;
            
        }

        public static function get_field_type($formid, $field_id) {
            $form = GFAPI::get_form($formid);
            $field = GFFormsModel::get_field($form, $field_id);
            return esigget('type', $field);
        }

        public static function prepareDisplay($display, $value, $label) {
            if ($display == "label" && !empty($label)) {
                return $label;
            } elseif ($display == "value" && !empty($value)) {
                return $value;
            } elseif ($display == "label_value" && !empty($label) && !empty($value)) {
                return $label . ": " . $value;
            }
        }
        
        public static function get_product($lead, $field, $field_id, $form,$display) {
            $value = RGFormsModel::get_lead_field_value($lead, $field);
           
            if($display =="value"){
              $label= array_shift($value );
            }
            else {
               $label= array_shift($value ); 
            }
             
            $display_value = GFCommon::get_lead_field_display($field, $value, $lead['currency']);
            $ret_value = apply_filters('gform_entry_field_value', $display_value, $field, $lead, $form);
           
            return self::prepareDisplay($display, trim($ret_value,","), $label);
           
        }

        public static function get_html($entries, $field, $field_id, $forms) {
            $html = '';
            if (!empty($field->content)) {
                $content = GFCommon::replace_variables_prepopulate($field->content); // adding support for merge tags
                $content = do_shortcode($content); // adding support for short
                $html .= $content;
            }
               
            $html .= str_replace('{FIELD}', '', GF_Fields::get('html')->get_field_content(esigget($entries,$field_id), true, $forms));

            return $html;
        }
        
         public static function get_post_image($entries, $field, $field_id, $forms) {
            $value = RGFormsModel::get_lead_field_value($entries, $field);
            return trim($value,"|:");
            
        }


        public static function getRadio($lead, $field, $form, $enableChoiceValue, $display, $label) {
            $defaultValue = self::defaultValue($lead, $field, $form, false, 'text');
            $choices = esigget('choices', $field);

            if (is_array($choices) && $enableChoiceValue) {
                foreach ($choices as $options) {
                    $text = esigget("text", $options);
                    $value = esigget("value", $options);
                    if ($value == $defaultValue) {
                        return self::prepareDisplay($display, $value, $text);
                    }
                }
            }

            return $defaultValue;
        }

        public static function getCheckbox($lead, $field, $form, $enableChoiceValue, $display, $label, $option) {

            if (!$enableChoiceValue && $display == "value") {

                if ($option == "check") {
                    $value = RGFormsModel::get_lead_field_value($lead, $field);

                    if (is_array($value)) {
                        $items = '';
                        foreach ($value as $key => $item) {
                            if (!rgblank($item)) {
                                $items .= '<li><span style="font-size:16px;">&#10003;</span>' . $item . '</li>';
                            }
                        }
                    }
                    return "<ul class='esig-checkbox-tick'>$items</ul>";
                } else {

                    return self::defaultValue($lead, $field, $form, false, 'html');
                }
            }

            $defaultValue = self::defaultValue($lead, $field, $form, false, 'text');
            $defaultLabel = self::defaultValue($lead, $field, $form, true, 'text');

            return self::prepareDisplay($display, $defaultValue, $defaultLabel);
            
        }

        public static function getMultiSelect($lead, $field, $form, $enableChoiceValue, $display, $option) {
            // return self::defaultValue($lead, $field, $form, false, 'html');
            if (!$enableChoiceValue && $display == "label") {
                return self::defaultValue($lead, $field, $form, false, 'html');
            }

            $defaultValue = self::defaultValue($lead, $field, $form, false, 'text');

            $currentValues = json_decode($defaultValue);
            $result = '';

            if (!is_array($currentValues)) {
                return false;
            }

            if ($enableChoiceValue && $display == "label") {
                foreach ($field->choices as $choice) {
                   
                    foreach ($currentValues as $item) {
                        if (RGFormsModel::choice_value_match($field, $choice, $item)) {
                            $result .= $choice['text'] . ', ';
                        }
                    }
                }
            }
            if ($enableChoiceValue && $display == "label_value") {
                foreach ($field->choices as $choice) {
                    foreach ($currentValues as $item) {
                        if (RGFormsModel::choice_value_match($field, $choice, $item)) {
                            $result .= $choice['text'] . ": " .$item . ', ';
                        }
                    }
                }
            }
            if ($enableChoiceValue && $display == "value") {
                foreach ($currentValues as $key => $value) {

                    $result .= $value . ', ';
                }
            }
            return substr($result, 0, strlen($result) - 2);
        }

        public static function remove_map_it($result) {
            return true;
        }

        public static function get_address($lead, $field, $field_id, $form) {
            add_filter("gform_disable_address_map_link", array(__CLASS__, "remove_map_it"), 10, 1);
            $value = RGFormsModel::get_lead_field_value($lead, $field);
            $display_value = GFCommon::get_lead_field_display($field, $value, $lead['currency']);
            return apply_filters('gform_entry_field_value', $display_value, $field, $lead, $form);
        }
        
         public static function get_number($lead, $field, $field_id, $form) {
           
               $format = esigget('numberFormat', $field);
               $value = RGFormsModel::get_lead_field_value($lead, $field);
               return GFCommon::format_number( $value,  $format,rgar( $lead, 'currency' ),true);
        }

        public static function defaultValue($lead, $field, $form, $use_text = false, $format = 'html') {
            $value = RGFormsModel::get_lead_field_value($lead, $field);
            $display_value = GFCommon::get_lead_field_display($field, $value, $lead['currency'], $use_text, $format);
            return apply_filters('gform_entry_field_value', $display_value, $field, $lead, $form);
        }

        public static function get_default_value($lead, $field, $form, $field_id, $display, $option) {

            //make a condition to check input field
            $type = esigget('type', $field);
            $label = isset($field->label) ? $field->label : false;
           
            if(empty($type) && !array_key_exists($field_id,$lead)){
                return false;
            }
        
            if(empty($type) && array_key_exists($field_id,$lead)){
                $type = "fieldDeleted";
            }
            
            
            switch ($type):
                case "product":
                    return self::get_product($lead, $field, $field_id, $form,$display);
                case "html":
                    return self::get_html($lead, $field, $field_id, $form);
                case "post_image":
                    return self::get_post_image($lead, $field, $field_id, $form);    
                case "address":
                    return self::get_address($lead, $field, $field_id, $form);
                case "number":
                    return self::get_number($lead, $field, $field_id, $form);    
                case "radio":
                    $enableChoiceValue = esigget("enableChoiceValue", $field);
                    return self::getRadio($lead, $field, $form, $enableChoiceValue, $display, $label);
                case "checkbox":
                    $enableChoiceValue = esigget("enableChoiceValue", $field);
                    return self::getCheckbox($lead, $field, $form, $enableChoiceValue, $display, $label, $option);
                case "select":
                    $enableChoiceValue = esigget("enableChoiceValue", $field);
                    return self::getRadio($lead, $field, $form, $enableChoiceValue, $display, $label);
                case "multiselect":
                    $enableChoiceValue = esigget("enableChoiceValue", $field);
                    return self::getMultiSelect($lead, $field, $form, $enableChoiceValue, $display, $option);
                case "fieldDeleted":
                    return self::displayDeletedField($field_id,$form,$lead) ;
                    break;  
                default:
                    $value = RGFormsModel::get_lead_field_value($lead, $field);
                    $display_value = GFCommon::get_lead_field_display($field, $value, $lead['currency']);
                    $ret_value = apply_filters('gform_entry_field_value', $display_value, $field, $lead, $form);
                    return self::prepareDisplay($display, $ret_value, $label);
            endswitch;
        }

        /**
         *  Display a deleted gravity field . If user deletes  a field from gravity form . It will still 
         *  Grab value from our database and display. For already submitted form.
         *  @param  int $fieldId  It is a Gravity Form field id  
         *  @param  int $form   It is a Gravity Form objet 
         *  @param  array $lead    It is gravity entry array . 
         * 
         *  @return {string/bool} | Value of deleted field entry.  
         */

        public static function displayDeletedField($fieldId,$formId,$lead){
           
             if(array_key_exists($fieldId,$lead)){

                  $fieldValue = esigget($fieldId,$lead);
                  return $fieldValue; 

             }
             return false;
        }

        public static function display_value($display, $document_id) {

            $display_type = ESIG_GF_SETTINGS::get_display_feed($document_id);
            if ($display_type == "underline") {
                return "<u>" . $display . "</u>";
            } else {
                return $display;
            }
        }

    }

    

    

    

    

    

    

    

    

    

    

    

endif;